%% 30/04/2021 - A baseband model of a downlink LTE system, for use with SNR  produced by noise calculations. Developed by Dr Stephen Henthorn based on work from Mathworks, used in Concurrent Multiband Direct RF Sampling Receivers. Please attribute when used for further work (CC BY-NC 4.0)
clear all
close all

%% LTE parameters
params = struct();          % Initialize the parameter structure
params.NDLRB = 96;         % 20 MHz bandwidth
params.CellRefP = 1;        % Cell reference signals on the first two ports
params.TotSubframes = 1;    % Number of subframes generated at one time
params.PDSCH.PRBSet = (0:params.NDLRB-1)'; % Full allocation
params.PDSCH.TargetCodeRate = 1/3; % The target code rate
params.PDSCH.TxScheme = 'Port0'; % Transmission mode 0
params.PDSCH.NLayers = 1;          % 1 layer transmission
params.PDSCH.Modulation = '16QAM';  % Modulation scheme
params.PDSCH.NSCID = 0;            % Scrambling identity
params.PDSCH.NTxAnts = 1;          % 1 transmit antenn
params.PDSCH.RVSeq = 0;            % Disable HARQ process
params.PDSCH.W = lteCSICodebook(params.PDSCH.NLayers,...
                        params.PDSCH.NTxAnts,0).'; % Precoding matrix  
params.gridSize = lteDLResourceGridSize(params); 
params.cfg.NRxAnts = 1;           % Receive antennas
% Configure channel estimator (values default in simulator)
params.cec.PilotAverage = 'TestEVM';                   % Type of pilot symbol averaging
params.cec.FreqWindow = 41;    % Frequency window size in REs
params.cec.TimeWindow = 27;    % Time window size in REs
params.cec.InterpType = 'cubic';  % 2D interpolation type
params.cec.InterpWindow = 'Centered'; % Interpolation window type
params.cec.InterpWinSize = 1;  % Interpolation window size
% Now use lteRMCDL to populate other parameter fields
fullParams = lteRMCDL(params);
clear params

%% Other parameters
%Define SNRs (may be loaded from calculation, may be preset values)
% snr=0:10; %Define a range of SNRs
load('Band4NyqSingle');snr=snrdB; %Previously calculated SNRs loaded in

maxSubframes=5000; %Maximum number of subframes
numErrs=50; %Number of CRC errors to give a valid result

wavPowdB=-36.3552; %Average power of downlink waveform generated

if  size(fullParams.PDSCH.Modulation,1) > 1 %Preallocate crc according to the number of codewords
    crc = false(length(snr),maxSubframes,2);
else
    crc = false(length(snr),maxSubframes,1);
end

bler=-ones(length(snr),1); %Preallocate
evm=-ones(length(snr),1);

%% Baseband simulation
for snrInd=1:length(snr) %Looping through SNRs
    
    for subframeInd=0:maxSubframes-1
        params.NSubframe = subframeInd; %Start subframe index
        
        if  size(fullParams.PDSCH.Modulation,1) > 1 %Generate right amount of random data
        fullParams.PDSCH.Modulation = cellstr(fullParams.PDSCH.Modulation)';
        data = [{randi([0 1], fullParams.PDSCH.TrBlkSizes(1, mod(fullParams.NSubframe, 10) +1), 1)} ...
            {randi([0 1], fullParams.PDSCH.TrBlkSizes(2, mod(fullParams.NSubframe, 10) +1), 1)}];
        else
            data= {randi([0 1], fullParams.PDSCH.TrBlkSizes(mod(fullParams.NSubframe, 10) +1), 1)};
        end
        
        [dlWaveform, symsIn] = lteRMCDLTool_SH(fullParams,data); % Generate the waveform using the full parameter set 'fullParams'

        varN=10.^((wavPowdB-snr(snrInd))/10)/2; %Variance of noise required for given SNR
        n=sqrt(varN)*(randn(length(dlWaveform),1)+1j*randn(length(dlWaveform),1)); %n is random complex noise
        rxWaveform=dlWaveform+n; %Additive noise

        rxgrid = lteOFDMDemodulate(fullParams, rxWaveform); %OFDM demodulate

        [hEst, nEst] = lteDLChannelEstimate(fullParams, fullParams.PDSCH, fullParams.cec, rxgrid); %Channel estimation

        [cw,symsOut]= PDSCHDecode_wrapper_SH(fullParams, rxgrid, hEst, nEst); %Decode rxgrid to recover codewords and symbols

        [rxbits, crc(snrInd,subframeInd+1,:)] = DLSCHDecode_wrapper(fullParams, cw); %Extract received bits and CRC from decode
        
        if sum(crc(snrInd,:,:))>numErrs-1
            break; %Stop when numErrs CRC checks come back as erros
        end
    end
    evm(snrInd)=lteEVM(symsIn,symsOut).RMS;
    bler(snrInd)=sum(sum(crc(snrInd,:,:)))/(subframeInd+1); %Calculate block error rate for given SNR
    save TempNyqSingle4 evm bler snrInd pBoIdBm snrdB
    if bler(snrInd)<0.01
        break; %Stop whole cycle when bler reaches <1% for number of subframes sent
    end
end
figure(1)
% semilogy(snr,bler) %Use when SNR defined
semilogy(pBoIdBm,bler) %Use when SNR calculated from received power